//
//  ProcessTest.cpp
//
//  Created  by Setsu on 6/26/10.
//  Modified by Setsu on May 30, 2013.
//  Copyright 2010-2013 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/Process.h>
#include <Poco/Pipe.h>
#include <Poco/PipeStream.h>
#include <Poco/StreamCopier.h>
#include <Poco/Format.h>

#include <string>
#include <vector>
#include <iostream>

#include "ScopedLogMessage.h"
#include "PrepareConsoleLogger.h"

//----------------------------------------
//	const
//----------------------------------------
#if defined(POCO_OS_FAMILY_WINDOWS)
const std::string kCommandOptionTable[][2] = {	{"date.bat",	""},
												{"ls.bat",		"/b"}};
#else
const std::string kCommandOptionTable[][2] = {	{"date",	"-u"},
												{"ls",		"-la"}};
#endif

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	PrepareConsoleLogger logger(Poco::Logger::ROOT, Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("ProcessTest ", "start", "end");

	for(std::size_t i=0; i<sizeof(kCommandOptionTable)/sizeof(kCommandOptionTable[0]); ++i)
	{
		msg.Message(Poco::format(" launch \"%s %s\"", kCommandOptionTable[i][0], kCommandOptionTable[i][1]));

		try
		{
			std::vector<std::string> args;
			args.push_back(kCommandOptionTable[i][1]);
			Poco::Pipe outPipe;
			Poco::ProcessHandle ph = Poco::Process::launch(kCommandOptionTable[i][0], args, 0, &outPipe, 0);
			Poco::PipeInputStream istr(outPipe);
			Poco::StreamCopier::copyStream(istr, std::cout);
			int rc = ph.wait();
			msg.Message(Poco::format("  return code = %d", rc));
		}
		catch (Poco::SystemException& exc)
		{
			std::cout << exc.displayText() << std::endl;
		}
	}

	return 0;
}
